<?php

use \Drupal\field\Field;

/*
 * implements hook_tokens().
 */
function mcapi_tokens($type, $tokens, array $data = array(), array $options = array()) {
  if ($type != 'mcapi' || !array_key_exists('mcapi', $data) || !is_object($data['mcapi'])) return array();
  $replacements = $replaced = array();
  $transaction = &$data[$type];
  module_load_include('inc', 'mcapi');
  mcapi_transaction_process_vars($replaced, $transaction, array('#showlinks' => in_array('links', $tokens)));

  foreach ($tokens as $name => $original) {
    $replacements[$original] = $replaced[$name];
  }
  //handle any entity references - tokens within tokens
  $token_service = \Drupal::token();
  if ($payer_tokens = $token_service->findWithPrefix($tokens, 'payer')) {
    $replacements += $token_service->generate(
      'user',
      $payer_tokens,
      array('user' => $transaction->payer->value),
      $options
    );
  }
  if ($payee_tokens = $token_service->findWithPrefix($tokens, 'payee')) {
    $replacements += $token_service->generate(
      'user',
      $payee_tokens,
      array('user' => $transaction->payee->value),
      $options
    );
  }
  if ($creator_tokens = $token_service->findWithPrefix($tokens, 'creator')) {
    $replacements += $token_service->generate(
      'user',
      $creator_tokens,
      array('user' => $transaction->creator->value),
      $options
    );
  }
  if ($created_tokens = $token_service->findWithPrefix($tokens, 'created')) {
    $replacements += $token_service->generate(
      'created',
      $created_tokens,
      array('created' => $transaction->created->value),
      $options
    );
  }
  return $replacements;
}

/**
 * implements hook_token_info().
 */
function mcapi_token_info() {
  $info = array(
    'types' => array(
      'mcapi' => array(
        'name' => t('Transactions'),
        'description' => t('Tokens related to a community accounting transactions.'),
        'needs-data' => 'mcapi_transaction',
      )
    )
  );
  $definitions = \Drupal::entityManager()->getFieldDefinitions('mcapi_transaction');
  //chance to remove some tokens here using unset($definitions['uuid']).

  //this really should be more automated in core
  foreach ($definitions as $key => $def) {
    $info['tokens']['mcapi'][$key] = array(
      'name' => $def->getLabel(),
      'description' => $def->getDescription()
    );
  }
  //add a couple more tokens for virtual fields
  $info['tokens']['mcapi']['links'] =  array(
    'name' => t('Links'),
    'description' => t("Links to transaction operations"),
  );
  $info['tokens']['mcapi']['url'] =  array(
    'name' => t('URL'),
    'description' => t("The url of the transaction"),
  );

  return $info;
}